<?php
/*
Plugin Name: Shortbus
Plugin URI: http://wordpress.org/extend/plugins/shortbus/
Description: Quickly and easily manage shortcodes.
Version: 1.1.1
Author: Matt Gibbs
Author URI: http://uproot.us/
License: GPL
Copyright: Matt Gibbs
*/
define('SHORTBUS_VERSION', '1.1.1');

class Shortbus
{
    /*---------------------------------------------------------------------------------------------
     * Verify the database, check Shortbus version, add necessary hooks
     *
     * @author Matt Gibbs
     * @since 1.0
     ---------------------------------------------------------------------------------------------*/
    function init() {
        global $wpdb;

        // Create the table if needed
        $wpdb->query("CREATE TABLE IF NOT EXISTS `{$wpdb->prefix}shortbus` (
        `id` int unsigned AUTO_INCREMENT PRIMARY KEY,
        `name` varchar(128),
        `content` mediumtext)");

        // Update the wp_option value
        $db_version = get_option('shortbus_version');
        if (false === $db_version) {
            add_option('shortbus_version', SHORTBUS_VERSION);
        }
        elseif (version_compare($db_version, SHORTBUS_VERSION, '<')) {
            update_option('shortbus_version', SHORTBUS_VERSION);
        }

        // Add hooks
        add_action('admin_init', array($this, 'help_box'));
        add_action('admin_menu', array($this, 'admin_menu'));
        add_action('admin_head', array($this, 'admin_scripts'));
        add_shortcode('sb', array($this, 'shortcode'));
        add_shortcode('shortbus', array($this, 'shortcode'));
        add_action('wp_ajax_shortbus', array($this, 'handle_ajax'));
        add_filter('widget_text', 'do_shortcode');
    }

    /*---------------------------------------------------------------------------------------------
     * Retrieve the shortcode content
     *
     * @author Matt Gibbs
     * @since 1.0
     ---------------------------------------------------------------------------------------------*/
    function shortcode($atts) {
        global $wpdb;

        $atts = (object) $atts;
        if (false === empty($atts->name)) {
            $row = $wpdb->get_row("SELECT content FROM `{$wpdb->prefix}shortbus` WHERE name = '$atts->name' LIMIT 1");
            ob_start();
            echo '<div class="shortbus">' . eval('?>' . $row->content) . '</div>';
            return ob_get_clean();
        }
    }

    /*---------------------------------------------------------------------------------------------
     * Add the submenu under "Pages"
     *
     * @author Matt Gibbs
     * @since 1.0
     ---------------------------------------------------------------------------------------------*/
    function admin_menu() {
        add_submenu_page('tools.php', 'Shortcodes', 'Shortcodes', 'edit_pages', 'shortbus', array($this, 'admin_page'));
    }

    /*---------------------------------------------------------------------------------------------
     * Standardize ajax responses
     *
     * @author Matt Gibbs
     * @since 1.1.1
     ---------------------------------------------------------------------------------------------*/
    function json_response($status = 'ok', $status_message = null, $data = null) {
        if (empty($status_message)) {
            $status_message = '<p>' . $status_message . '</p>';
        }
        return json_encode(
            array(
                'status' => $status,
                'status_message' => $status_message,
                'data' => $data,
            )
        );
    }

    /*---------------------------------------------------------------------------------------------
     * Save the shortcode data
     *
     * @author Matt Gibbs
     * @since 1.0
     ---------------------------------------------------------------------------------------------*/
    function handle_ajax() {
        global $wpdb;

        $id = (int) $_POST['id'];
        $method = $_POST['method'];

        if ('load' == $method) {
            $row = $wpdb->get_row("SELECT content FROM `{$wpdb->prefix}shortbus` WHERE id = '$id' LIMIT 1");
            echo $this->json_response('ok', null, $row->content);
        }
        elseif ('add' == $method) {
            $name = trim($_POST['name']);

            if (!preg_match('/^[A-Za-z0-9\-_ ]+$/', $name)) {
                echo $this->json_response('error', 'Please use only alphanumeric characters, spaces, hyphens, and underscores.');
            }
            else {
                $wpdb->insert($wpdb->prefix . 'shortbus', array('name' => $name));
                echo $this->json_response('ok', 'Shortcode added.', array('id' => $wpdb->insert_id));
            }
        }
        elseif ('edit' == $method) {
            $content = mysql_real_escape_string(stripslashes($_POST['content']), $wpdb->dbh);
            $wpdb->query("UPDATE `{$wpdb->prefix}shortbus` SET content = '$content' WHERE id = '$id' LIMIT 1");
            echo $this->json_response('ok', 'Shortcode saved.');
        }
        elseif ('delete' == $method) {
            $wpdb->query("DELETE FROM `{$wpdb->prefix}shortbus` WHERE id = '$id' LIMIT 1");
        }
        elseif ('export' == $method) {
            $data = $wpdb->get_results("SELECT name, content FROM `{$wpdb->prefix}shortbus` ORDER BY id");
            echo $this->json_response('ok', null, json_encode($data));
            
        }
        elseif ('import' == $method) {
            
        }
        die();
    }

    /*---------------------------------------------------------------------------------------------
     * Add javascript to the admin header
     *
     * @author Matt Gibbs
     * @since 1.0
     ---------------------------------------------------------------------------------------------*/
    function admin_scripts() {
?>
<script type="text/javascript">
var shortcode_id;

jQuery(function() {

    // change select box value
    jQuery(".sb-option").live("click", function() {
        shortcode_id = jQuery(this).attr("rel");
        jQuery("#sb-select-value").html(jQuery(this).html());
        jQuery("#sb-select-box").removeClass("active");
        jQuery("#sb-select-popup").toggle();
        jQuery("#shortcode-content").val("");
        var data = {
            method: "load",
            action: "shortbus",
            id: jQuery(this).attr("rel")
        };
        if ("" == data.id) {
            jQuery("#shortcode-area").hide();
            jQuery("#shortcode-intro").show();
        }
        else {
            jQuery("#save-area").hide();
            jQuery("#loading-area").show();
            jQuery.post(ajaxurl, data, function(response) {
                jQuery("#shortcode-content").val(response.data);
                jQuery("#shortcode-intro").hide();
                jQuery("#shortcode-area").show();
                jQuery("#loading-area").hide();
                jQuery("#save-area").show();
            }, "json");
        }
    });

    // select box
    jQuery("#sb-select-box").click(function() {
        jQuery(this).toggleClass("active");
        jQuery("#sb-select-popup").toggle();
    });

    // filter select box options
    jQuery("#sb-filter-input").keyup(function() {
        var keyword = jQuery(this).val();
        if ("" == keyword) {
            jQuery(".sb-option").show();
        }
        else {
            jQuery(".sb-option").each(function() {
                var html = jQuery(this).html();
                var regex = new RegExp(keyword, "i");
                if (0 <= html.search(regex)) {
                    jQuery(this).show();
                }
                else {
                    jQuery(this).hide();
                }
            });
        }
    });

    // add
    jQuery("#add-shortcode").click(function() {
        var data = {
            method: "add",
            action: "shortbus",
            name: jQuery("#shortcode-name").val()
        };

        if ("" == data.name) return;
        jQuery.post(ajaxurl, data, function(response) {
            if ("ok" == response.status) {
                jQuery("#shortcode-name").val("");
                jQuery("#shortcode-content").val("");
                jQuery("#sb-select-options").append('<div class="sb-option" rel="'+response.data.id+'">'+data.name+'</div>');
                jQuery("#sb-select-value").html(data.name);
                jQuery("#shortcode-intro").hide();
                jQuery("#shortcode-area").show();
                shortcode_id = response.data.id;
            }
            jQuery("#shortcode-response").html("<p>"+response.status_message+"</p>");
            jQuery("#shortcode-response").show();
        }, "json");
    });

    // edit
    jQuery("#edit-shortcode").click(function() {
        var data = {
            method: "edit",
            action: "shortbus",
            id: shortcode_id,
            content: jQuery("#shortcode-content").val()
        };
        jQuery.post(ajaxurl, data, function(response) {
            jQuery("#shortcode-response").html("<p>"+response.status_message+"</p>");
            jQuery("#shortcode-response").show();
        }, "json");
    });

    // delete
    jQuery("#delete-shortcode").click(function() {
        if (confirm("Are you sure you want to delete this shortcode?")) {
            var data = {
                method: "delete",
                action: "shortbus",
                id: shortcode_id
            };
            jQuery.post(ajaxurl, data, function(response) {
                jQuery("#shortcode-area").hide();
                jQuery("#shortcode-intro").show();
                jQuery("#sb-select-value").html("Select one");
                jQuery(".sb-option[rel="+shortcode_id+"]").remove();
                shortcode_id = null;
            }, "json");
        }
    });

    // export
    jQuery("#export").click(function() {
        var data = { method: "export", action: "shortbus" };
        jQuery.post(ajaxurl, data, function(response) {
            alert(response.data);
        }, "json");
    });
});
</script>
<?php
    }

    /*---------------------------------------------------------------------------------------------
     * Use the contextual help box
     *
     * @author Matt Gibbs
     * @since 1.1.1
     ---------------------------------------------------------------------------------------------*/
    function help_box() {
        ob_start();
?>
        <div><strong>To add a shortcode...</strong></div>
        <div>Enter a shortcode name into the text box and click <strong>Add New</strong>.</div>
        <div style="margin-top:15px"><strong>To edit a shortcode...</strong></div>
        <div>Select an existing shortcode from the dropdown.</div>
        <div style="margin-top:15px"><strong>Usage</strong></div>
        <div>Add <span class="code">[sb name="My Shortcode"]</span> within your pages, posts, or widgets.</div>
<?php
        $help_text = ob_get_clean();
        add_contextual_help('tools_page_shortbus', $help_text);
    }

    /*---------------------------------------------------------------------------------------------
     * Build the shortcode management page
     *
     * @author Matt Gibbs
     * @since 1.0
     ---------------------------------------------------------------------------------------------*/
    function admin_page() {
        global $wpdb;
?>
<link href="<?php echo WP_PLUGIN_URL; ?>/shortbus/style.css" rel="stylesheet" type="text/css" />
<div class="wrap">
	<div id="icon-edit-pages" class="icon32 shortbus-icon"><br></div>
    <h2>Shortcodes</h2>

    <div id="shortcode-response" class="updated hidden"></div>

    <div style="margin:15px 0">
        <div id="sb-select">
            <div id="sb-select-box">
                <div id="sb-select-value">Select one</div>
            </div>
            <div id="sb-select-popup">
                <div id="sb-filter">
                    <input type="text" id="sb-filter-input" value="" /> Search
                </div>
                <div id="sb-select-options">
                    <div class="sb-option" rel="">Select one</div>
<?php
        $results = $wpdb->get_results("SELECT id, name FROM `{$wpdb->prefix}shortbus` ORDER BY name ASC");
        foreach ($results as $result) {
?>
                    <div class="sb-option" rel="<?php echo $result->id; ?>"><?php echo $result->name; ?></div>
<?php
        }
?>
                </div>
            </div>
        </div>
        - or -
        <input id="shortcode-name" type="text" title="Enter a name for your shortcode" value="" />
        <a id="add-shortcode" class="button">Add New</a>
    </div>
    <div id="shortcode-area" class="hidden">
        <div><textarea id="shortcode-content"></textarea></div>
        <div id="save-area">
            <input type="submit" class="button-primary" id="edit-shortcode" value="Save Edits" />
            or <a id="delete-shortcode" href="javascript:;">Delete</a>
        </div>
        <div id="loading-area">
            <span id="loading"></span> Loading, please wait...
        </div>
    </div>
    <div id="shortcode-intro">
        <div><strong>Need help?</strong></div>
        <div>Click on "Help" at the top right of this screen.</div>
        <div style="margin-top:15px"><strong>Migrate your shortcodes</strong></div>
        <!--<div><a id="import" href="javascript:;">Import</a> and <a id="export" href="javascript:;">export</a> within seconds.</div>-->
        <div>Coming soon.</div>
        <div style="margin-top:15px"><strong>What do you think of this plugin?</strong></div>
        <div>
            Please
            <a href="http://wordpress.org/extend/plugins/shortbus/" target="_blank">rate it</a>,
            <a href="http://wordpress.org/tags/shortbus?forum_id=10" target="_blank">send feedback</a>, or
            <a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=66U2MHS4XDRRY" target="_blank">consider donating</a>.
        </div>
    </div>
</div>
<?php
    }
}

/*---------------------------------------------------------------------------------------------
 * Initialize Shortbus
 *
 * @author Matt Gibbs
 * @since 1.0
 ---------------------------------------------------------------------------------------------*/
$sb = new Shortbus();
$sb->init();
